@@ -1196,6 +1196,30 @@ xref_meta_src(Head, Called, _) :-
     Extra is Arity - 1,
     arg(1, Head, G),
     Called = [G+Extra].
+xref_meta_src(Head, Called, _) :-
+    predicate_property(user:Head, meta_predicate(Meta)),
+    !,
+    Meta =.. [_|Args],
+    meta_args(Args, 1, Head, Called).
+
+meta_args([], _, _, []).
+meta_args([H0|T0], I, Head, [H|T]) :-
+    xargs(H0, N),
+    !,
+    arg(I, Head, A),
+    (   N == 0
+    ->  H = A
+    ;   H = (A+N)
+    ),
+    I2 is I+1,
+    meta_args(T0, I2, Head, T).
+meta_args([_|T0], I, Head, T) :-
+    I2 is I+1,
+    meta_args(T0, I2, Head, T).
+
+xargs(N, N) :- integer(N), !.
+xargs(//, 2).
+xargs(^, 0).
 
 apply_pred(call).                               % built-in
 apply_pred(maplist).                            % library(apply_macros)
