@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2018, University of Amsterdam
+    Copyright (c)  2002-2020, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
     All rights reserved.
@@ -57,6 +57,7 @@
 :- use_module(http_wrapper).
 :- use_module(http_path).
 
+:- autoload(library(uri), [uri_resolve/3]).
 
 :- predicate_options(http_server/2, 2,
                      [ port(any),
@@ -125,6 +126,9 @@ self-signed SSL certificate.
     http:create_pool/1,
     http:schedule_workers/1.
 
+:- meta_predicate
+    thread_repeat_wait(0).
+
 %!  http_server(:Goal, :Options) is det.
 %
 %   Create a server at Port that calls Goal for each parsed request.
@@ -221,7 +225,7 @@ make_socket(Port, _:Options0, Options) :-
     tcp_socket(Socket),
     tcp_setopt(Socket, reuseaddr),
     tcp_bind(Socket, Port),
-    tcp_listen(Socket, 5),
+    tcp_listen(Socket, 64),
     make_addr_atom('httpd', Port, Queue),
     Options = [ queue(Queue),
                 tcp_socket(Socket)
@@ -413,6 +417,7 @@ accept_server3(Goal, Options) :-
 accept_server3(Goal, Options) :-
     memberchk(tcp_socket(Socket), Options),
     memberchk(queue(Queue), Options),
+    debug(http(connection), 'Waiting for connection', []),
     tcp_accept(Socket, Client, Peer),
     debug(http(connection), 'New HTTP connection from ~p', [Peer]),
     thread_send_message(Queue, tcp_client(Client, Goal, Peer)),
@@ -467,17 +472,23 @@ connect(Address) :-
 %   hook http:schedule_workers/1 to extend  the   worker  pool. This
 %   predicate can be used by accept_hook/2.
 
+http_enough_workers(Queue, _Why, _Peer) :-
+    message_queue_property(Queue, waiting(_0)),
+    !,
+    debug(http(scheduler), '~D waiting for work; ok', [_0]).
 http_enough_workers(Queue, Why, Peer) :-
     message_queue_property(Queue, size(Size)),
     (   enough(Size, Why)
-    ->  true
+    ->  debug(http(scheduler), '~D in queue; ok', [Size])
     ;   current_server(Port, _, _, Queue, _, _),
-        catch(http:schedule_workers(_{port:Port,
-                                      reason:Why,
-                                      peer:Peer,
-                                      waiting:Size,
-                                      queue:Queue
-                                     }),
+        Data = _{ port:Port,
+                  reason:Why,
+                  peer:Peer,
+                  waiting:Size,
+                  queue:Queue
+                },
+        debug(http(scheduler), 'Asking to reschedule: ~p', [Data]),
+        catch(http:schedule_workers(Data),
               Error,
               print_message(error, Error))
     ->  true
@@ -578,11 +589,11 @@ resize_pool(Queue, Size) :-
 %   stops.
 
 http_worker(Options) :-
-    thread_at_exit(done_worker),
+    debug(http(scheduler), 'New worker', []),
+    prolog_listen(this_thread_exit, done_worker),
     option(queue(Queue), Options),
     option(max_idle_time(MaxIdle), Options, infinite),
-    repeat,
-      thread_idle(get_work(Queue, Message, MaxIdle), long),
+    thread_repeat_wait(get_work(Queue, Message, MaxIdle)),
       debug(http(worker), 'Waiting for a job ...', []),
       debug(http(worker), 'Got job ~p', [Message]),
       (   Message = quit(Sender)
@@ -920,6 +931,67 @@ create_pool(Pool) :-
     thread_pool_create(Pool, 10, []).
 
 
+		 /*******************************
+		 *         WAIT POLICIES	*
+		 *******************************/
+
+:- meta_predicate
+    thread_repeat_wait(0).
+
+%!  thread_repeat_wait(:Goal) is multi.
+%
+%   Acts as `repeat,  thread_idle(Goal)`,  choosing   whether  to  use a
+%   `long` or `short` idle time based on the average firing rate.
+
+thread_repeat_wait(Goal) :-
+    new_rate_mma(5, 1000, State),
+    repeat,
+      update_rate_mma(State, MMA),
+      long(MMA, IsLong),
+      (   IsLong == brief
+      ->  call(Goal)
+      ;   thread_idle(Goal, IsLong)
+      ).
+
+long(MMA, brief) :-
+    MMA < 0.05,
+    !.
+long(MMA, short) :-
+    MMA < 1,
+    !.
+long(_, long).
+
+%!  new_rate_mma(+N, +Resolution, -State) is det.
+%!  update_rate_mma(!State, -MMA) is det.
+%
+%   Implement _Modified Moving  Average_  computing   the  average  time
+%   between requests as an exponential moving averate with alpha=1/N.
+%
+%   @arg Resolution is the time resolution  in 1/Resolution seconds. All
+%   storage is done in integers to avoid  the need for stack freezing in
+%   nb_setarg/3.
+%
+%   @see https://en.wikipedia.org/wiki/Moving_average
+
+new_rate_mma(N, Resolution, rstate(Base, 0, MaxI, Resolution, N, 0)) :-
+    current_prolog_flag(max_tagged_integer, MaxI),
+    get_time(Base).
+
+update_rate_mma(State, MMAr) :-
+    State = rstate(Base, Last, MaxI, Resolution, N, MMA0),
+    get_time(Now),
+    Stamp is round((Now-Base)*Resolution),
+    (   Stamp > MaxI
+    ->  nb_setarg(1, State, Now),
+        nb_setarg(2, State, 0)
+    ;   true
+    ),
+    Diff is Stamp-Last,
+    nb_setarg(2, State, Stamp),
+    MMA is round(((N-1)*MMA0+Diff)/N),
+    nb_setarg(6, State, MMA),
+    MMAr is MMA/float(Resolution).
+
 
                  /*******************************
                  *            MESSAGES          *
