@@ -37,6 +37,7 @@
 #include <assert.h>
 #include <string.h>
 #include <math.h>
+#include <ctype.h>
 
 static int protocol_error(IOSTREAM *in, const char *id);
 static int unexpected_eof(IOSTREAM *in);
@@ -59,6 +60,7 @@ static atom_t ATOM_pairs;
 static atom_t ATOM_tagged_integer;
 static atom_t ATOM_dict_key;
 static atom_t ATOM_dict;
+static atom_t ATOM_prolog;
 
 static functor_t FUNCTOR_status1;
 static functor_t FUNCTOR_prolog1;
@@ -169,6 +171,36 @@ read_line(IOSTREAM *in, charbuf *cb)
 }
 
 
+static char *
+read_number_line(IOSTREAM *in, charbuf *cb)
+{ int maxlen = 100;
+
+  for(;maxlen-- > 0;)
+  { int c = Sgetcode(in);
+
+    if ( c == -1 )
+      return unexpected_eof(in),NULL;
+    if ( c == '\r' )
+    { add_byte_charbuf(cb, 0);
+      if ( Sgetcode(in) != '\n' )
+	return newline_expected(in),NULL;
+      return cb->base;
+    }
+    if ( c == '\n' )
+    { add_byte_charbuf(cb, 0);
+      return cb->base;
+    }
+    if ( (c >= '0' && c <= '9') ||
+	 ((c == '-' || c == '?') && cb->here == cb->base) )
+      add_byte_charbuf(cb, c);
+    else
+      return protocol_error(in, "integer_expected"),NULL;
+  }
+
+  return protocol_error(in, "integer_maxlen"),NULL;
+}
+
+
 		 /*******************************
 		 *	      ERRORS		*
 		 *******************************/
@@ -265,7 +297,7 @@ read_number(IOSTREAM *in, charbuf *cb, long long *vp)
 { long long v;
   char *s, *end;
 
-  if ( !(s=read_line(in, cb)) )
+  if ( !(s=read_number_line(in, cb)) )
     return FALSE;
   v = strtoll(s, &end, 10);
   if ( *end )
@@ -279,7 +311,7 @@ static int
 read_length(IOSTREAM *in, charbuf *cb, long long *vp)
 { char *s;
 
-  if ( !(s=read_line(in, cb)) )
+  if ( !(s=read_number_line(in, cb)) )
     return FALSE;
   if ( cb->base[0] == '?' )
   { *vp = LEN_STREAM;
@@ -570,6 +602,8 @@ type_name(redis_type *type)
     case T_FLOAT:           return "float";
     case T_RATIONAL:        return "rational";
     case T_NUMBER:          return "number";
+    case T_PAIRS:           return "pairs";
+    case T_DICT:            return "dict";
     default:	            return "unknown";
   }
 }
@@ -595,6 +629,7 @@ str_is_number(size_t len, const char *data, enum ntype *nt)
     }
     if ( *data >= '0' && *data <= '9' )
     { isnum = TRUE;
+      signok = FALSE;
       continue;
     }
     if ( *data == '.' && isnum && !has_dot && !israt )
@@ -654,6 +689,23 @@ is_tagged_integer(term_t t)
 }
 
 
+static int
+return_type_error(term_t error, const char *etype, redis_type *type,
+		  size_t len, char *data)
+{ term_t t;
+
+  return ( (t=PL_new_term_ref()) &&
+	   PL_unify_chars(t, PL_STRING|REP_UTF8, len, data) &&
+	   PL_unify_term(error,
+			 PL_FUNCTOR_CHARS, "error", 2,
+			   PL_FUNCTOR_CHARS, etype, 2,
+			     PL_CHARS, type_name(type),
+			     PL_TERM, t,
+			   PL_VARIABLE) );
+}
+
+
+
 static int
 fixup_number(term_t t, term_t message, term_t error,
 	     size_t len, char *data, redis_type *type)
@@ -688,17 +740,9 @@ fixup_number(term_t t, term_t message, term_t error,
   }
 
   if ( rc )
-  { rc = PL_unify(message, t);
-  } else
-  { rc = ( PL_put_variable(t) &&
-	   PL_unify_chars(t, PL_STRING|REP_UTF8, len, data) &&
-	   PL_unify_term(error,
-			 PL_FUNCTOR_CHARS, "error", 2,
-			   PL_FUNCTOR_CHARS, error_name, 2,
-			     PL_CHARS, type_name(type),
-			     PL_TERM, t,
-			   PL_VARIABLE) );
-  }
+    rc = PL_unify(message, t);
+  else
+    rc = return_type_error(error, error_name, type, len, data);
 
   return rc;
 }
@@ -755,8 +799,7 @@ unify_bulk(term_t message, term_t error, size_t len, char *data, redis_type *typ
 	     PL_put_term_from_chars(t, REP_ISO_LATIN_1, len, data) &&
 	     fixup_number(t, message, error, len, data, type) );
   } else
-  { assert(0);
-    return FALSE;
+  { return return_type_error(error, "type_error", type, len, data);
   }
 }
 
@@ -785,11 +828,16 @@ redis_read_stream(IOSTREAM *in, term_t message, term_t error, term_t push,
       break;
     case '+':
       if ( !(s=read_line(in, &cb)) )
-	rc = FALSE;
-      else
+      { rc = FALSE;
+      } else
+      { char *q;
+
+	for(q=s; *q; q++)
+	  *q = tolower(*q);
 	rc = PL_unify_term(message,
 			   PL_FUNCTOR, FUNCTOR_status1,
-			     PL_UTF8_STRING, s);
+			     PL_UTF8_CHARS, s);
+      }
       break;
     case ':':
     { long long v;
@@ -1033,8 +1081,13 @@ static foreign_t
 redis_read_msg(term_t from, term_t msgin, term_t msgout,
 	       term_t error, term_t push)
 { IOSTREAM *in;
-  redis_type rt[7] = { { .kind     = T_TEXT,
-                         .pltype   = PL_STRING,
+  redis_type rt[7] = { { .kind     = T_AUTO
+                       },
+		       { .kind     = T_TEXT,
+                         .pltype   = PL_ATOM,
+                         .encoding = REP_UTF8
+                       },
+		       { .kind     = T_NUMBER,
                          .encoding = REP_UTF8
                        }
 		     };
@@ -1070,6 +1123,59 @@ redis_read_msg(term_t from, term_t msgin, term_t msgout,
     return rc;
   }
 
+  return FALSE;
+}
+
+
+static int
+expects_string(IOSTREAM *in, size_t len, const char *s)
+{ for(; len > 0; len--, s++)
+  { if ( Sgetc(in) != ((*s)&0xff) )
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
+
+static foreign_t
+redis_resync(term_t from, term_t Magic)
+{ IOSTREAM *in;
+  char *magic;
+  size_t len;
+  char slen[64];
+
+  if ( !PL_get_nchars(Magic, &len, &magic, CVT_STRING|CVT_INTEGER|CVT_EXCEPTION) )
+    return FALSE;
+  Ssnprintf(slen, sizeof(slen), "%d\r\n", len);
+
+  if ( PL_get_stream(from, &in, SIO_INPUT) )
+  { int rc;
+
+    for(;;)
+    { if ( Sgetc(in) == '$' &&
+	   expects_string(in, strlen(slen), slen) &&
+	   expects_string(in, len, magic) &&
+	   Sgetc(in) == '\r' &&
+	   Sgetc(in) == '\n' )
+      { rc = TRUE;
+	break;
+      }
+
+      if ( Sfeof(in) )
+      { rc = unexpected_eof(in);
+	break;
+      }
+    }
+
+    if ( rc )
+      rc = PL_release_stream(in);
+    else
+      PL_release_stream_noerror(in);
+
+    return rc;
+  }
+
   return FALSE;
 }
 
@@ -1079,7 +1185,7 @@ redis_read_msg(term_t from, term_t msgin, term_t msgout,
 		 *******************************/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Command is cmd(arg, ...), where arg is one of prolog(Term) or must be
+Command is cmd(arg, ...), where arg is one  of `Term as Type` or must be
 converted into a string.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -1206,6 +1312,23 @@ redis_write_stream(IOSTREAM *out, term_t message)
 	} else if ( PL_is_functor(arg, FUNCTOR_colon2) )
 	{ if ( !redis_write_key(out, arg) )
 	    return FALSE;
+	} else if ( PL_is_functor(arg, FUNCTOR_as2) )
+	{ term_t ta;
+	  atom_t name;
+	  size_t arity;
+
+	  if ( (ta=PL_new_term_ref()) &&
+	       PL_get_arg(2, arg, ta) &&
+	       PL_get_name_arity(ta, &name, &arity) &&
+	       PL_get_arg(1, arg, arg) )
+	  { if ( name == ATOM_prolog && arity == 0 )
+	    { if ( !redis_write_typed(out, arg, 'T', CVT_WRITE_CANONICAL) )
+		return FALSE;
+	    } else
+	    { return PL_domain_error("redis_type", ta);
+	    }
+	  } else
+	    return FALSE;
 	} else if ( PL_is_functor(arg, FUNCTOR_prolog1) )
 	{ _PL_get_arg(1, arg, arg);
 	  if ( !redis_write_typed(out, arg, 'T', CVT_WRITE_CANONICAL) )
@@ -1289,6 +1412,7 @@ install_redis4pl(void)
   MKATOM(tagged_integer);
   MKATOM(dict_key);
   MKATOM(dict);
+  MKATOM(prolog);
 
   MKFUNCTOR(as, 2);
   MKFUNCTOR(status, 1);
@@ -1302,4 +1426,5 @@ install_redis4pl(void)
   PL_register_foreign("redis_read_msg",		  5, redis_read_msg,	       0);
   PL_register_foreign("redis_write_msg",	  2, redis_write_msg,	       0);
   PL_register_foreign("redis_write_msg_no_flush", 2, redis_write_msg_no_flush, 0);
+  PL_register_foreign("$redis_resync",            2, redis_resync,             0);
 }
