@@ -319,7 +319,7 @@ PL_get_text__LD(term_t l, PL_chars_t *text, int flags ARG_LD)
     encodings[1] = ENC_WCHAR;
     encodings[2] = ENC_UNKNOWN;
 
-    if ( (flags&CVT_WRITEQ) == CVT_WRITEQ )
+    if ( (flags&CVT_WRITEQ) )
       wflags = PL_WRT_QUOTED|PL_WRT_NUMBERVARS;
     else if ( (flags&CVT_WRITE_CANONICAL) )
       wflags = PL_WRT_QUOTED|PL_WRT_IGNOREOPS|PL_WRT_NUMBERVARS;
@@ -912,6 +912,8 @@ PL_mb_text(PL_chars_t *text, int flags)
       }
     }
 
+    PL_free_text(text);
+
     text->length   = sizeOfBuffer(b)-1;
     text->text.t   = baseBuffer(b, char);
     text->encoding = target;
