@@ -432,10 +432,10 @@ bindConst__LD(Word p, word c ARG_LD)
 
 
 static inline word
-consPtr__LD(void *p, word ts ARG_LD)
+consPtr__LD(void *p, uintptr_t base, word ts ARG_LD)
 { uintptr_t v = (uintptr_t) p;
 
-  v -= LD->bases[ts&STG_MASK];
+  v -= base;
   DEBUG(CHK_SECURE, assert(v < MAXTAGGEDPTR && !(v&0x3)));
   return (v<<5)|ts;
 }
@@ -452,6 +452,35 @@ valFloat__LD(word w ARG_LD)
 }
 #endif
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+linkValI(p)  is  an  inlined  version  of  linkVal()  that  assumes  the
+dereferenced value of `p` lives on  the   global  stack. It has no error
+checking (unless compiled for debugging) and fetches the base address of
+the global stack only once.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static inline word
+linkValI__LD(Word p ARG_LD)
+{ word w = *p;
+  uintptr_t gb = LD->bases[STG_GLOBAL];
+
+  while(isRef(w))
+  { //p = unRef(w);
+    p = (Word)valPtrB(w,gb);
+    w = *p;
+  }
+
+  DEBUG(CHK_ATOM_GARBAGE_COLLECTED, assert(w != ATOM_garbage_collected));
+
+  if ( !needsRef(w) )
+  { return w;
+  } else
+  { // return makeRefG(p);
+    DEBUG(0, assert(p<(Word)lBase));
+    return consPtrB(p, gb, TAG_REFERENCE|STG_GLOBAL);
+  }
+}
+
 static inline int
 is_signalled(ARG1_LD)
 { return HAS_LD && unlikely((LD->signal.pending[0]|LD->signal.pending[1]) != 0);
@@ -624,26 +653,34 @@ normally 0). longToPointer() does the inverse operation.
 
 static inline uintptr_t
 pointerToInt(void *ptr)
-{ uintptr_t p   = (uintptr_t) ptr;
+{
+#if SIZEOF_VOIDP == 8
+  return (uintptr_t) ptr;
+#else
+  uintptr_t p   = (uintptr_t) ptr;
   uintptr_t low = p & 0x3L;
 
-  p -= GD->heap_base;
   p >>= 2;
   p |= low<<(sizeof(uintptr_t)*8-2);
 
   return p;
+#endif
 }
 
 
 static inline void *
 intToPointer(uintptr_t p)
-{ uintptr_t low = p >> (sizeof(uintptr_t)*8-2);
+{
+#if SIZEOF_VOIDP == 8
+  return (void*) p;
+#else
+  uintptr_t low = p >> (sizeof(uintptr_t)*8-2);
 
   p <<= 2;
   p |= low;
-  p += GD->heap_base;
 
   return (void *) p;
+#endif
 }
 
 #endif /*PL_INLINE_H_INCLUDED*/
