@@ -85,6 +85,15 @@ typedef struct tbl_component
   trie		       *leader;			/* Leading variant */
 } tbl_component;
 
+typedef struct tbl_status
+{ tbl_component *scc;
+  int		 hsc;
+  int		 iac;
+} tbl_status;
+
+void save_tabling_status(tbl_status *stat);
+void restore_tabling_status(tbl_status *stat);
+
 
 		 /*******************************
 		 *	   TABLE WORKLIST	*
@@ -223,6 +232,7 @@ typedef struct trie_array
 
 #define TP_MONOTONIC	(0x0001)	/* Monotonic tabling */
 #define TP_SHARED	(0x0002)	/* Shared tabling */
+#define TP_OPAQUE	(0x0004)	/* Declared opaque */
 
 typedef struct table_props
 { unsigned int	flags;			/* TP_* flags */
